# -*- coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/


require APP_ROOT + "/models/journal_entry"


# 勘定科目
class AccountTitle < ActiveRecord::Base
  belongs_to :user

  ACTYPE_NAMES = [nil, '資産', '負債', '純資産', '営業収益', '営業費用', 
                  'その他の収益', 'その他の費用']
  ACTYPE_BS = [nil, true, true, true, false, false, false, false]

  CFTYPE_NAMES = [nil, '営業活動', '投資活動', '財務活動', '現金同等物']

  Cache = {}
  # ユーザを都度指定すると面倒なので、ヘルパ関数を用意する
  def self.find_item(id_, user)
    raise ArgumentError if !id_
    raise TypeError if !user.is_a?(User)

    Cache[id_] ||= find(id_, :conditions => ['user_id = ?', user.id])
    return Cache[id_]
  end

  # @return 借方科目（資産、費用）=> +1
  def sign
    return [nil, 1, -1, -1, -1, 1, -1, 1][ac_type]
  end

  def bs?
    return AccountTitle::ACTYPE_BS[ac_type]
  end

  def pl?
    return !bs?
  end

  # @return 仕訳がなく、未払金でもなければtrue
  def can_delete? user
    raise TypeError if !user.is_a?(User)

    j = JournalEntry.find :first, 
              :conditions => ["user_id = ? AND (dr_code = ? OR cr_code = ?)",
                              user.id, id, id]
    return false if j
    return false if user.user_setting.payable_item == id

    return true
  end
end

# 追加可能な科目一覧
# プルダウンメニュー
def get_sel_active_items(user, name, default)
  raise TypeError if !user.is_a?(User)

  ac = AccountTitle.find(:all, 
                         :conditions => ['user_id = ? and active <> 0',
                                         user.id],
                         :order => 'ac_type, sort')
  ac = ac.select {|item| yield item} if block_given?
  return get_sel_acitems_sub(ac, name, default)
end

def get_sel_active_items_with_zero(user, name, default)
  raise TypeError if !user.is_a?(User)

  ac = AccountTitle.find(:all, 
                         :conditions => ['user_id = ? and active <> 0',
                                         user.id],
                         :order => 'ac_type, sort')
  ac = ac.select {|item| yield item} if block_given?
  return get_sel_acitems_sub(ac, name, default, true)
end

# 購買伝票用の科目一覧（簡便）
def get_sel_purchase_items(user, name, default)
  raise TypeError if !user.is_a?(User)
  ac = AccountTitle.find(:all, 
                         :conditions => ['user_id = ? and active <> 0',
                                         user.id],
                         :order => 'ac_type, sort')
  ac = ac.select{|item| 
    if [1, 5, 7].include?(item.ac_type) && item.cf_type != 4 
      true
    else
      false
    end
  }
  return get_sel_acitems_sub(ac, name, default, false)
end

# 仕訳データがある科目一覧
def get_sel_viewable_items(user, name, default)
  raise TypeError if !user.is_a?(User)

  ac = AccountTitle.find_by_sql([<<EOF, user.id, user.id])
(SELECT dr_code, account_titles.* FROM journal_entries JOIN account_titles ON journal_entries.dr_code = account_titles.id WHERE journal_entries.user_id = ?) 
UNION 
(SELECT cr_code, account_titles.* FROM journal_entries JOIN account_titles ON journal_entries.cr_code = account_titles.id WHERE journal_entries.user_id = ?)
EOF
  return get_sel_acitems_sub(ac, name, default)
end

# HTML selectおよび中身を生成する
# 勘定科目をグルーピングする
def get_sel_acitems_sub(item_ary, name, default, with_zero = false)
  s = "<select name=\"#{name}\">\n"
  s << "  <option value=\"0\">　　〃</option>\n" if with_zero
  last_ac_type = nil
  item_ary.each {|item|
    if item.ac_type != last_ac_type
      s << "</optgroup>\n" if last_ac_type
      s << "<optgroup label=\"#{AccountTitle::ACTYPE_NAMES[item.ac_type]}\">\n"
      last_ac_type = item.ac_type
    end
    selected = (default == item.id) ? " selected" : ""
    s << "  <option#{selected} value=\"#{item.id}\">" +
         CGI.escapeHTML(item.name) + "</option>\n"
  }
  s << "</optgroup>\n" if last_ac_type
  s << "</select>\n"
  return Amrita::SanitizedString[s]
end
