
require APP_ROOT + '/models/account_title'

# 仕訳
class JournalEntry < ActiveRecord::Base
  belongs_to :user
  belongs_to :invoice
  belongs_to :payment_amount

  # @override
  def validate
    if !amount || amount == 0
      errors.add 'amount', '金額が0の仕訳は作れません。'
    end
  end

  # 指定期間の純利益金額
  # start_day  開始日  nilのときは初日から
  # end_day 終了日（この日を含む）
  def self.get_profit_amount user, start_day, end_day
    profit = 0
    if !start_day
      cond = ['journal_entries.user_id = ? and date <= ?', user.id, end_day]
    else
      cond = ['journal_entries.user_id = ? and date BETWEEN ? AND ?',
                                 user.id, start_day, end_day]
    end

    ary = JournalEntry.find :all,
                 :select => 'journal_entries.*, drac.ac_type AS dr_type, crac.ac_type AS cr_type',
                 :joins => 'JOIN account_titles AS drac ON dr_code = drac.id JOIN account_titles AS crac ON cr_code = crac.id',
                 :conditions => cond

    ary.each {|r|
      if !AccountTitle::ACTYPE_BS[r.dr_type.to_i]
        profit -= r.amount
      end
      if !AccountTitle::ACTYPE_BS[r.cr_type.to_i]
        profit += r.amount
      end
    }
    return profit
  end

  # start_day の前日の残高 (B/S科目のみ, P/L科目は常に0)
  # 戻り値
  #     借方残がプラス。（負債・資本はプラスのときに戻り値はマイナス）
  def self.carrying_amount user, item, start_day
    if item.bs? 
      balance = JournalEntry.find(:first, 
                        :conditions => ['user_id = ? and dr_code = ? and date < ?',
                                        user.id, item.id, start_day],
                        :select => "sum(amount) as amo").amo.to_i - 
                JournalEntry.find(:first, 
                        :conditions => ['user_id = ? and cr_code = ? and date < ?',
                                        user.id, item.id, start_day],
                        :select => "sum(amount) as amo").amo.to_i
    else
      balance = 0
    end

    # 未処分利益
    if item.id == user.user_setting.unappropriated_item
      balance -= get_profit_amount user, nil, start_day - 1
    end

    return balance
  end
end

# 同じ仕訳か？                                  
def same_entry?(x, y)
  return false if !x || !y
  if x.date == y.date && x.entry_no == y.entry_no
    true
  else 
    false
  end
end
