#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/


# 予算実績対比 budgetary comparison

require "pcview.rb"
require APP_ROOT + '/models/budget_detail'
require APP_ROOT + '/models/journal_entry'
require "journal-data.rb"
require LIBRARY_DIR + "/ordered_set"


class StartForm < Wafu::Action
  def execute af
    bday = UserSetting.get_ac_month(app.user)[0]
    eday = (bday >> 12) - 1

    @by, @bm, @bd = [bday.year, bday.mon, bday.day]
    @ey, @em, @ed = [eday.year, eday.mon, eday.day]

    render "actual1", :layout => "application"
  end
end


class BudgetaryComparisonView < Wafu::Action

  def lookup_amount ary, item
    raise TypeError if !item.is_a?(AccountTitle)

    ary.each {|r|
      return r.amount.to_i if r.account_title_id == item.id
    }
    return nil
  end
  private :lookup_amount

  def execute af
    if af["b"]
      @bday = Date.parse af["b"]
      @eday = Date.parse af["e"]
    else
      @bday = Date.new af["byear"].to_i, af["bmon"].to_i, af["bday"].to_i
      @eday = Date.new af["eyear"].to_i, af["emon"].to_i, af["eday"].to_i
    end

=begin
    op_date = user.user_setting.opening_date
    begin_date = op_date + 1 if begin_date <= op_date
    end_date = op_date + 1 if end_date <= op_date
=end

    @budgets = Budget.find :all, 
                   :conditions => ["user_id = ? AND (date BETWEEN ? AND ?)",
                                   app.user.id, @bday, @eday],
                   :order => "date, id"

    adays = (@budgets.collect do |budget| budget.date - 1 end) + [@eday]
    actuals = Account.new(app.user).get_fs_multi(adays, :bs => :occur)

    # 期首の現金同等物の残高
    if @budgets.size >= 1
      start_cash = 0
      actuals[0].each do |ac_code, amount|
        start_cash += amount if AccountTitle.find(ac_code).cf_type == 4
      end

      budget_cash = start_cash
      actual_cash = start_cash
    end

    # 表形式に整える
    sec = OrderedSet.new( lambda do |x, y| 
                            if x[:ac].sort != y[:ac].sort
                              return x[:ac].sort < y[:ac].sort ? true : false
                            else
                              x[:ac].id < y[:ac].id ? true : false
                            end
                          end )

    @headers = [ [], [] ]
    # 営業収益 営業費用 その他の収支 資産負債 期首資金残
    @sections = [ sec, sec.dup, sec.dup, sec.dup, [] ]

    tbl_col = 0
    (0 .. @budgets.size - 1).each do |col|
      @headers[0] << {:day => @budgets[col].date, :colspan => 1}
      @headers[1] << "予算"
      @sections[4] << budget_cash
      @budgets[col].budget_details.each do |line|
        add_col_amount tbl_col, line.account_title, line.amount
        budget_cash -= line.amount if line.account_title.cf_type != 4
      end
      tbl_col += 1

      if !actuals[col + 1].empty?
        @headers[0].last[:colspan] = 3
        @headers[1] << "実績" << "差異"

        @sections[4] << actual_cash
        actuals[col + 1].each do |code, amount|
          add_col_amount tbl_col, (ac = AccountTitle.find(code)), amount
          actual_cash -= amount if ac.cf_type != 4
        end
        tbl_col += 1

        # 差異
        titles = AccountTitle.find :all, 
                           :conditions => ["user_id = ?", app.user.id]
        titles.each do |title|
          ba = (lookup_amount @budgets[col].budget_details, title).to_i
          aa = actuals[col + 1][title.id].to_i
          if ba != 0 || aa != 0
            add_col_amount tbl_col, title, aa - ba
          end
        end
        @sections[4] << @sections[4][-1] - @sections[4][-2]
        tbl_col += 1
      end
    end

    render "actual-view", :layout => "application"
  end

private
  def add_col_amount tbl_col, account_title, amount
    return if account_title.cf_type == 4 # 現金同等物
        
    section = @sections[ type_idx account_title ]
    v = {:ac => account_title, :amounts => []}
    if !(i = section.index v)
      section << v 
    else
      v = section[i]
    end

    # 空白を補う
    (v[:amounts].length .. tbl_col - 1).each do v[:amounts] << 0 end
        
    v[:amounts] << amount # ここでは符号を変えない
  end

  # 勘定科目から各セクションに区分する
  def type_idx title
    raise "internal error" if title.cf_type == 4

    [nil, 3, 3, 3, 0, 1, 2, 2][title.ac_type]
  end

end



(PCView.new :actions => {
  "*" => [Wafu::ActionForm, StartForm, nil],
  "view" => [Wafu::ActionForm, BudgetaryComparisonView, nil]
 } ).run
