#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/

# 予算の一覧

require "pcview.rb"
require APP_ROOT + "/models/budget"
require "journal-data.rb"


class StartForm < Wafu::Action
  def execute af
    bday = UserSetting.get_ac_month(app.user)[0]
    eday = (bday >> 12) - 1

    @by, @bm, @bd = [bday.year, bday.mon, bday.day]
    @ey, @em, @ed = [eday.year, eday.mon, eday.day]

    render "budgets-list1", :layout => "application"
  end
end


class Form < Wafu::Action
  # 指定の期間内の予算を並べて表示
  # 例えば、
  #     1w 1w 1w 1w 1m 1m 3m 3m 3m 1y
  def execute af
    if af["b"]
      @bday = Date.parse af["b"]
      @eday = Date.parse af["e"]
    else
      @bday = Date.new af["byear"].to_i, af["bmon"].to_i, af["bday"].to_i
      @eday = Date.new af["eyear"].to_i, af["emon"].to_i, af["eday"].to_i
    end

    @budgets = Budget.find :all, 
                 :conditions => ["user_id = ? AND (date BETWEEN ? AND ?)", 
                                 app.user.id, @bday, @eday],
                 :order => "date, id"

    # 表の形にマッピングする。
    # 営業収益, 営業費用, その他収益/費用, 現金同等物以外のBS項目
    @sections = [ {}, {}, {}, {} ]

    @budgets.each_with_index do |budget, col|
      budget.budget_details.each do |line|
        next if line.account_title.cf_type == 4  # 現金同等物

        section = @sections[ type_idx line.account_title ]
        section[ line.account_title.id ] ||= [] 

        # 空白を補う
        (section[line.account_title.id].length .. col - 1).each do 
          section[line.account_title.id] << 0
        end

        section[line.account_title.id] << line.amount
      end
    end

    if @budgets.size >= 1
      # 期首の現金同等物の残高
      @start_cash = 0

      accounting = Account.new app.user
      bal = accounting.get_fs_multi [@budgets.first.date - 1]
      bal[0].each do |ac_code, amount|
        @start_cash += amount if AccountTitle.find(ac_code).cf_type == 4
      end
    end

    render "budgets-list", :layout => "application"
  end

private
  # 勘定科目から各セクションに区分する
  def type_idx title
    raise "internal error" if title.cf_type == 4

    [nil, 3, 3, 3, 0, 1, 2, 2][title.ac_type]
  end
end


(PCView.new :actions => {
  "view" => [Wafu::ActionForm, Form, nil], 
  "*" => [Wafu::ActionForm, StartForm, nil]
} ).run
