#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/

# 債務の消し込み


require 'payment-common.rb'

class FirstData < Wafu::ActionForm
  def setup
    date = Date.today
    @mon = date.mon
    @day = date.day
    @cash_ac = nil

    unpaid_invoices = 
        Invoice.find :all,
            :select => "invoices.*, amo.paid",
            :joins => "LEFT JOIN (SELECT invoice_id, SUM(amount) AS paid FROM payment_amounts GROUP BY invoice_id) amo ON invoices.id = amo.invoice_id",
            :conditions => ["invoices.user_id = ? AND ((amo.paid IS NULL) OR invoices.amount_total <> amo.paid)", app.user.id]

    @data = []
    unpaid_invoices.each {|r|
      @data << {
        :invoice => r,
        :paid => r.paid.to_i
      }
    }
    @lines = @data.size
  end
  attr_reader :mon, :day, :cash_ac, :lines, :data
  attr_reader :payment
end


# レコードを生成
class PaymentCreate < Wafu::Action
  def execute(af)
    user_setting = app.user.user_setting
    begin_day, end_day = user_setting.get_addable_period
    d = get_full_day(af.mon.to_i, af.day.to_i, begin_day)
    raise if !d || d < begin_day || d > end_day

    Payment.transaction {
      payment = Payment.new(:user => @app.user, :date => d)
      payment.save!

      entry_no = (user_setting.entry_no_counter += 1)
      user_setting.save!
      cash_ac = AccountTitle.find(af.cash_ac)
      af.data.each {|line|
        if line[:pay].to_i != 0
          inv = line[:invoice]
          raise if inv.user.id != @app.user.id
          pa = PaymentAmount.new(:payment => payment, 
                                 :invoice => inv, 
                                 :amount => line[:pay].to_i)
          pa.save!
          payment.payment_amounts << pa

          e = JournalEntry.new(:user => @app.user,
                               :date => d,
                               :entry_no => entry_no,
                               :dr_code => user_setting.payable_item,
                               :cr_code => cash_ac.id,
                               :amount => pa.amount,
                               :remarks => inv.partner.name,
                               :comment => "支払い " + inv.journal_entries.first.comment,
                               :payment_amount => pa)
          e.save!
        end
      }
      raise if payment.payment_amounts.size == 0 # 1件もない
    }

    app.set_flash("notice", "支払いを登録しました。")
    app.redirect_to "credit.rb"
  end
end


PCView.new('actions' => {
  "*" => [FirstData, InvoiceList, nil],
  "pay" => [PaymentAF, PaymentCreate, InvoiceList]
}).run()
