# -*- coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/


require CONF_DIR + "/ac-conf.rb"

require "date"
require LIBRARY_DIR + "/cgisup.rb"
require LIBRARY_DIR + "/datesup.rb"
require LIBRARY_DIR + "/strstore.rb"
require APP_ROOT + '/models/journal_entry'


class Account
  def initialize user
    raise TypeError if !user.is_a?(User)

    @op_date = user.user_setting.opening_date
    @user = user
  end

  # 各日付までのB/S金額，P/L金額を求める。最初の日付分は，B/Sのみ。
  # 
  # @param [Array] day_ary    日付 (B/S日) の配列。
  # @param [Hash] options
  # @option options [Symbol] :bs   B/S項目を発生金額とするときは:occur
  #
  # @return 列の配列. 列は勘定科目idをキーとするハッシュ.
  def get_fs_multi day_ary, options = {}
    raise TypeError if !day_ary.is_a?(Array)
    raise ArgumentError if day_ary.size < 1

    records = JournalEntry.find(:all, 
                                :conditions => ['user_id = ? and date <= ?',
                                                @user.id, day_ary.last],
                                :order => 'date')

    result = []
    ridx = 0
    day_ary.each {|d|
      raise TypeError if !d.is_a?(Date)

      fs = {} # 列
      if !(options[:bs] == :occur)
        # B/Sは残高
        if result.size > 0
          result.last.each {|k, v|
            fs[k] = v if AccountTitle.find_item(k, @user).bs?
          }
        end
      end

      while ridx < records.size && records[ridx].date <= d
        r = records[ridx]
        fs[r.dr_code] = (fs[r.dr_code] || 0) + r.amount
        fs[r.cr_code] = (fs[r.cr_code] || 0) - r.amount
        ridx += 1
      end
      result << fs
    }

    return result
  end
end
