#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# PC用　元帳

require "pcview.rb"
require APP_ROOT + '/models/invoice'
require APP_ROOT + '/models/payment_amount'
require APP_ROOT + '/models/account_title'


# 期間選択（画面）
class LedgerForm < Wafu::Action
  def execute(af)
    begin_date, end_date = UserSetting.get_ac_month(@app.user)
    end_date = [end_date, Date.today].max
    item = @app.user.user_setting.ledger_last_item

    model = {
      :by => begin_date.year,
      :bm => begin_date.mon,
      :bd => begin_date.day,
      :ey => end_date.year,
      :em => end_date.mon,
      :ed => end_date.day,
      :aclist => get_sel_viewable_items(@app.user, "code", item)
    }
    @app.page_out("ledger.html", model)
  end
end

class LedgerFormReload < Wafu::Action
  def execute(af)
    model = {
      :by => af.by, :bm => af.bm, :bd => af.bd,
      :ey => af.ey, :em => af.em, :ed => af.ed
    }
    if af.error?
      model[:errors] = af.errors
    end
    ac_chooser = AcChooser.new(@app.user)
    model[:aclist] = ac_chooser.get_sel_acitems("code", af.accode)
    
    @app.page_out("ledger.html", model)
  end
end


class RangeCheck < DateRangeActionForm
  def validate()
    super()
    @accode = @app.cgi["code"].first.to_i
  end
  attr_reader :accode
end


class LedgerView < Wafu::Action
  # 指定された勘定科目について、指定期間の仕訳および開始時点の残高を得る
  def get_journals(user, begin_date, end_date, item)
    raise TypeError if !begin_date.is_a?(Date)
    raise TypeError if !end_date.is_a?(Date)
    raise TypeError if !item.is_a?(AccountTitle)
    return nil if begin_date > end_date
    return nil if @app.user.user_setting.opening_date > end_date

    balance = JournalEntry.carrying_amount app.user, item, begin_date
    entries = JournalEntry.find(:all, 
                  :conditions => ['user_id = ? and (dr_code = ? or cr_code = ?) and date between ? and ?', 
                                  @app.user.id, item.id, item.id, 
                                  begin_date, end_date],
                  :order => 'date, entry_no, id')
    return [balance, entries]
  end


  def form_out(cgi, begin_date, end_date, item, data)
    balance, ary = data
    new_begin, new_end = UserSetting.get_ac_month(@app.user)
    new_end = [new_end, Date.today].max
    
    model = {
      :notice => app.get_flash("notice"),
      :ac_sel => get_sel_viewable_items(@app.user, "code", item.id),
      :begin_date => format_date_long(begin_date),
      :end_date => format_date_long(end_date),
      :ac_name => CGI.escapeHTML(item.name),
      :total_name => (item.bs? ? "残高" : "累計"),
      
      :ledger => [],

      :by => new_begin.year.to_s,
      :bm => new_begin.mon.to_s,
      :bd => new_begin.day.to_s,
      :ey => new_end.year.to_s,
      :em => new_end.mon.to_s,
      :ed => new_end.day.to_s
    }

    if item.bs?
      model[:if_bs] = {:brought => decimal_format(balance * item.sign)}
      model[:ledger] << nil

      # 未処分利益
      if item.id == app.user.user_setting.unappropriated_item
        pl_alter = JournalEntry.get_profit_amount app.user, begin_date, end_date
      end
    end

    last_day = nil
    prev = nil
    ary.each_with_index {|r, idx|
      if same_entry?(prev, r)
        h = {
          :date => "",
          :num => "",
          :if_cur => nil
        }
      else
        h = {
          :date => Amrita::SanitizedString[format_date2(r.date, last_day)],
          :num => r.entry_no,
          :if_cur => get_edit_url(r, @fixed, "ledger.rb?action=view&amp;from=#{begin_date}&amp;to=#{end_date}&amp;code=#{item.id}"),
          :if_inv => r.invoice ? {:show => "invoices-show.rb?id=#{r.invoice.id}"} : nil
        }
      end
      h.update({
        :opponent => (if r.dr_code == item.id
                        AccountTitle.find_item(r.cr_code, @app.user).name
                      else
                        AccountTitle.find_item(r.dr_code, @app.user).name
                       end),
        :dr => (if r.dr_code == item.id
                   balance += r.amount
                   decimal_format(r.amount)
                 else
                   ""
                 end),
        :cr => (if r.cr_code == item.id
                   balance -= r.amount
                   decimal_format(r.amount)
                 else
                   ""
                 end),
        :remarks => CGI.escapeHTML(r.remarks.to_s),
        :comment => CGI.escapeHTML(r.comment.to_s),
        :balance => (if ary[idx + 1] && r.date == ary[idx + 1].date
                        ""
                      else
                        decimal_format(balance * item.sign)
                     end),
      })
      model[:ledger] << h
      last_day = r.date
      prev = r
    }
    if item.id == app.user.user_setting.unappropriated_item
      model[:ledger] << {
        :date => end_date,
        :num => '',
        :opponent => '当期純利益',
        :dr => '',
        :cr => decimal_format(pl_alter),
        :remarks => '',
        :comment => '',
        :balance => decimal_format(balance + pl_alter)
      }
    end
    @app.page_out("ledger2.html", model)
  end
  
  def execute(af)
    begin_date = af.begin_date
    end_date = af.end_date

    item = nil
    data = nil
    prof = @app.user.user_setting
    @fixed = prof.fixed_date
    op_date = prof.opening_date

    begin_date = op_date + 1 if begin_date <= op_date
    end_date = op_date + 1 if end_date <= op_date

    item = AccountTitle.find_item(@app.cgi["code"].first.to_i, @app.user)
    data = get_journals(@app.user, begin_date, end_date, item)
    raise if !data

    prof.ledger_last_item = item.id
    prof.save!

    form_out(@app.cgi, begin_date, end_date, item, data)
  end
end


PCView.new('actions' => {
      "*" => [nil, LedgerForm, nil],
      "view" => [RangeCheck, LedgerView, LedgerFormReload],
    }).run
