#!/home/netsphere/bin/ruby

# Q's Account Book
# Copyright (c) 2002-2008 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 携帯用　現金出納帳

require '../mobile_app.rb'
require APP_ROOT + '/models/account_title'
require APP_ROOT + '/models/journal_entry'
require LIBRARY_DIR + '/datesup'


class SetupAF < Wafu::ActionForm
  def setup
    @mode = app.cgi.get('mode')
    raise if mode.nil? || mode == ''

    now = Time.now
    now = Date.new(now.year, now.mon, now.day) - 1 if now.hour < 3
    @mon = now.mon; @day = now.day
  end
  attr_reader :mon, :day, :mcode, :data, :lines
  attr_reader :mode
end


# 編集画面
class CashEdit < MobileAction
  def execute af
    cash_codes = AccountTitle.find :all, 
            :conditions => ["user_id = ? AND active <> 0 AND cf_type = 4 AND name_short <> ''", app.user.id]
    ac_codes = AccountTitle.find :all,
            :conditions => ["user_id = ? AND active <> 0 AND cf_type <> 4 AND name_short <> ''", app.user.id]
    
    lines = 4 # magic
    model = {
      :title => (af.mode == "in" ? "入金" : "出金"),
      :mode => af.mode,
      :mcode => get_sel_acitems_sub(cash_codes, "mcode", nil),
      :mon => af.mon,
      :day => af.day, 
      :list => [],
      :lines => lines,
      :_session_id => app.session.session_id,
      :link_menu => link_to('メニューに戻る', :action => '../m.rb')
    }

    (1..lines).each {|i|
      h = {
        :accode => get_sel_acitems_sub(ac_codes, "ac#{i}", nil),
        :amountid => "cr#{i}",
        :remarks => "remarks-#{i}",
        :comment => "comment-#{i}",
      }
      model[:list] << h
    }
    app.page_out "cash.html", model
  end
end


class CreateAF < Wafu::ActionForm
  def setup
    set_cgi_attrs('mon' => String, 'day' => String, 
                  'mcode' => "int", 
                  'lines' => "int",
                  'mode' => String)
    @data = []
    (1..lines).each {|i|
      data << {
        :ac => app.cgi.get("ac#{i}"), 
        :amount => app.cgi.get("cr#{i}"),
        :remarks => CSConv.s2u(app.cgi.get("remarks-#{i}")),
        :comment => CSConv.s2u(app.cgi.get("comment-#{i}"))
      }
    }
  end
  attr_reader :mon, :day, :mcode, :data, :lines
  attr_reader :mode

  def validate
    err = []
    period = app.user.user_setting.get_addable_period
    if !(d = get_full_day(mon.to_i, day.to_i, period[0]))
      err << "日付が正しくありません。"
    elsif d < period[0] || d > period[1]
      err << "日付が追加可能期間の外です。"
    end
    return set_error(err)
  end
end


# 生成
class CashCreate < MobileAction
  def execute af
    prof = app.user.user_setting
    period = prof.get_addable_period

    day = get_full_day(af.mon.to_i, af.day.to_i, period[0])
    raise if !day || day < period[0] || day > period[1]

    JournalEntry.transaction {
      entry_no = (prof.entry_no_counter += 1)

      af.data.each_with_index {|line, i|
        amount = line[:amount].to_i
        next if amount == 0
        r = JournalEntry.new :user => app.user,
                  :date => day,
                  :entry_no => entry_no,
                  :dr_code => (af.mode == "in" ? af.mcode : line[:ac]),
                  :cr_code => (af.mode == "in" ? line[:ac] : af.mcode),
                  :amount => amount,
                  :remarks => line[:remarks],
                  :comment => line[:comment]
        raise if r.dr_code == r.cr_code
        r.save!
      }
      prof.save!
    }

    # 1日分を表示する。
    model = {
      :mon => day.mon,
      :day => day.day,
      :data => []
    }
    app.page_out("cash-create.html", model)
  end
end


MobileApp.new('actions' => {
  '*' => [SetupAF, CashEdit, nil],
  'create' => [CreateAF, CashCreate, CashEdit],
}).run

