
# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# ケータイ用アプリケーションクラス

class File
  def File.rel2abs(rel_path, this_file)
    File.expand_path(rel_path, File.dirname(File.expand_path(this_file)))
  end
end

require File.rel2abs('accounting_app', __FILE__)
require LIBRARY_DIR + '/csconv'


class MobileAction < Wafu::Action
  def link_to label, opt
    ary = []
    opt.each {|k, v|
      ary << "#{k}=#{v}" if k != :action
    }
    ary << "_session_id=#{app.session.session_id}"
    url = opt[:action] + '?' + ary.join('&amp;')
    r = %Q(<a href="#{url}">#{label}</a>)
    return Amrita::SanitizedString[r]
  end
end

class MobileApp < AccountingApp
  def initialize(opt_hash = {})
    opt = {
      "session_option" => {
        "tmpdir" => SESSION_DIR, 
        "holdtime" => SESSION_HOLD_TIME,
        "store_in_cookie" => "no"  # ケータイはURLにセッションIDを入れる
      },
    }
    opt.update(opt_hash)
    super(opt)
  end

  # @override
  # ダイジェストで認証
  def check_passwd()
    uid = cgi.get('u', nil)
    d = cgi.get('d', nil)
    if !uid || !d
      super  # 通常のログインフォームから
    else
      u = User.find :first, 
                    :conditions => ['login = ? AND crypted_password = ?', uid, d]
      if u
        return [Wafu::LOGON_SUCCESS, uid]
      else
        return [Wafu::LOGON_RETRY, nil]
      end
    end
  end

  def cgi_header_out()
    print cgi.header({'type' => 'text/html', 'charset' => 'Shift_JIS'})
  end

  def internal_error_message_out()
    cgi_header_out()
    html = <<EOF
<html>
<head>
  <title>内部エラー発生</title>
</head>
<body>
</table></ul></dl>
<h1>内部エラー発生</h1>
<p>予期しないエラーが発生しました。ご迷惑をお掛けします。
EOF
    html << "<p>" << CGI.escapeHTML($!.inspect) << "\n"
    html << "<p>"
    $@.each {|x| html << CGI.escapeHTML(x) << "<br>\n"}
    html << "<hr>\n" << CGI.escapeHTML(cgi.params.inspect)
    print CSConv.u2s(html)
  end

  def login_form(mes)
    cgi_header_out()
    s = <<EOF
<html>
<head>
  <title>ログイン</title>
</head>
<body>
<h1>ログイン</h1>
EOF
    if mes == Wafu::LOGON_RETRY
      s += "ユーザIDまたはパスワードが違います。\n"
    elsif mes == Wafu::LOGON_EXPIRE
      s += "セッション維持時間を越えました。再ログインが必要です。\n"
    end
    s += <<EOF
<form method="get" action="#{AppConfig[:site_top_url]}m.rb">
ユーザID:<input type="text" size="4" mode="alphabet" name="user"><br>
パスワード:<input type="text" size="4" mode="alphabet" name="pwd"><br>
<input type="submit" name="login" value="ログイン">
</form>
</body>
</html>
EOF
    print CSConv.u2s(s)
  end

  def footer_out()
    print <<EOF
</body>
</html>
EOF
  end

  def page_out(filename, mapping)
    raise SecurityError if filename =~ /\.\./
                                      
    tmpl = Amrita::TemplateText.new(File.read("#{APP_ROOT}/templates/mobile/" + filename).untaint)
    tmpl.amrita_id = "tid"
=begin
    mapping = model.dup.update({
      :theme => theme_url(user),
      :navbar => Amrita::SanitizedString[get_navigation_bar()],
      :navbar_modal => Amrita::SanitizedString[get_navbar_modal()],
      :footer => Amrita::SanitizedString[get_footer()],
      :sitename => CGI.escapeHTML(SITE_NAME),
    })
=end
    cgi_header_out()
    utext = ""
    tmpl.expand(utext, mapping)
    print CSConv.u2s(utext)
  end
end
