# -*- coding:utf-8 -*-

class File
  def File.rel2abs(rel_path, this_file)
    File.expand_path(rel_path, File.dirname(File.expand_path(this_file)))
  end
end

require File.rel2abs("./banner.rb", __FILE__)
require CONF_DIR + "/ac-conf.rb"
require LIBRARY_DIR + "/cgisup.rb"


SITE_MAP = {
  'index.rhtml' => {:title => "Home"},
  'doc/index.rhtml' => {:title => "Documentation"},
  "doc/privacy.rhtml" => {:title => "個人情報保護方針"},
  "doc/donation.rhtml" => {:title => "寄付"},

  # Features
  "doc/features.rhtml" => {:title => "機能・操作方法"},
  'doc/setup.rhtml' => {:title => '準備'},
  'doc/profile.rhtml' => {:title => 'プロフィール・画面テーマ'},
  "doc/master_data.rhtml" => {:title => "勘定科目マスタ"},
  'doc/cash.rhtml' => {:title => '日常の入出金'},
  "doc/accounts_payable.rhtml" => {:title => "後払い"},
  "doc/general_ledger.rhtml" => {:title => "振替伝票, 元帳, 仕訳帳"},
  'doc/report.rhtml' => {:title => "財務諸表、予算管理"},

  'doc/history.rhtml' => {:title => "更新履歴 2002-2008"},
  'doc/history-2.rhtml' => {:title => '更新履歴'},

  # How-to
  'doc/booking.rhtml' => {:title => 'かんたん簿記'},

  # Download
  'doc/accountbook.rhtml' => {:title => "Q's Web Account Book"},
  'doc/qab-releases.rhtml' => {:title => "ダウンロード"},
  "doc/qab3-install.rhtml" => {:title => "インストール"},

  # dev
  'doc/development.rhtml' => {:title => "開発に協力"},

}

class Navigation
  def initialize(key_)
    @key = key_
  end

  def title()
    if @key == "index.rhtml"
      return "無料で使える『シンプルWeb家計簿』"
    else
      return "シンプルWeb家計簿: #{SITE_MAP[@key][:title]}"
    end
  end

  def head_content()
    return <<EOF
  <meta http-equiv="Content-Language" content="ja">
  <meta http-equiv="Content-Script-Type" content="text/javascript">
  <meta http-equiv="Content-Style-Type" content="text/css">
  <link rel="start" href="#{AppConfig[:site_top_url]}">
  <link rev="made" href="mailto:vzw00011@nifty.ne.jp">
  <title>#{title()}</title>
  <base target="_top">
  <link rel="stylesheet" href="#{AppConfig[:site_top_url]}lib/deliciously_blue/style.css" type="text/css" media="screen,projection" />
  <link rel="stylesheet" href="#{AppConfig[:site_top_url]}lib/deliciously_blue/print.css" type="text/css" media="print" />
  <link rel="stylesheet" href="#{AppConfig[:site_top_url]}default.css" />
EOF
  end

  def link_to(to_key)
    if @key == to_key 
      SITE_MAP[to_key][:title]
    else
      "<a href=\"#{to_key}\">#{SITE_MAP[to_key][:title]}</a>"
    end
  end

  def top_menu()
    s = <<EOF
  <div id="page">
    
EOF
    if @key != "index.rhtml"
      s << '<div id="header" style="height:40px">'
    else
      s << <<EOF 
    <div id="header">
      <div id="headerimg">
        <h1><a href="#{AppConfig[:site_top_url]}">シンプルWeb家計簿</a></h1>
        <div class="description">お金が貯まる！　無料で使えるオンライン家計簿</div>
      </div>
EOF
    end
    s << <<EOF
      <ul id="nav">
        <li class="page_item"><a href="#{AppConfig[:site_top_url]}" title="Home">Home</a></li>
        <li class="page_item"><a href="#{AppConfig[:site_top_url]}doc/features.rhtml" title="機能">機能・操作方法</a></li>
        <li class="page_item"><a href="#{AppConfig[:site_top_url]}doc/accountbook.rhtml" title="Download">Download</a></li>
        <li class="page_item"><a href="#{AppConfig[:site_top_url]}doc/development.rhtml" title="開発に協力">開発に協力</a></li>
        <li class="page_item"><a href="#{AppConfig[:site_top_url]}doc/donation.rhtml" title="寄付">寄付</a></li>
      </ul>   
    </div>
    <!-- end header -->

<!-- begin index -->
    <div id="content">
EOF
    return s
  end

  def bottom()
    b = Banner.new
    return <<EOF
#{b.banner_footer(@key)}
    </div><!-- end content -->

<!-- sidebar -->
<div id="sidebar">
  <ul>
    <li>
<form action="#{AppConfig[:site_top_url]}login.rb" method="post">
<table style="background-color:#e0e0e0;border:outset 1px;margin-top:1em">
  <tr><th nowrap align="right">ユーザID: 
    <td><input type="text" name="user" style="width:8em">
  <tr><th nowrap align="right">パスワード:
    <td><input type="password" name="pwd" style="width:8em">
  <tr><td colspan="2" align="center">
    <input type="submit" name="login" value="ログイン">
</table>
</form>

<p align="left"><a href="#{AppConfig[:site_top_url]}sendpass.rb">[ユーザID・パスワードを忘れた場合]</a>
<p align="left"><a href="#{AppConfig[:site_top_url]}signup.rb">[新規ユーザ登録...]</a>
    </li>

    <li>
      <h2>[PR]</h2>
#{b.banner_sidebar(@key)}
    </li>
  </ul>
</div>
<!-- end sidebar -->

<!-- footer -->
<div id="footer">
  <div class="footer-about">
    <h4>About</h4>
<p><a href="#{AppConfig[:site_top_url]}doc/privacy.rhtml">個人情報保護方針</a>
<p><a href="http://www.nslabs.jp/profile.rhtml">管理者について</a>
  </div>

  <hr class="clear" />
</div> <!-- end footer -->
<p align="center">Copyright (c) HORIKAWA Hisashi. All rights reserved. - Theme designed by <a href="http://www.jdavidmacor.com/templates/">J David Macor.com</a>
</div>
EOF
=begin 
    return b.banner_body(@key, nil) << <<EOF
<hr>
<div align="right">
シンプルWeb家計簿 http://www.nslabs.jp/ac/<br>
Copyright (c) HORIKAWA Hisashi. All rights reserved.
</div>
EOF
=end
  end
end

def begin_post title, anchor_name = nil
  t = anchor_name ? 
    "<a href=\"##{anchor_name}\" name=\"#{anchor_name}\">_</a>#{CGI.escapeHTML(title)}</a>" :
    CGI.escapeHTML(title)
               
  return <<EOF
<div class="post" style="margin-top:2em">
  <h2>#{t}</h2>
  <div class="post-content">
EOF
end

def end_post 
  return <<EOF
  </div>
</div>
EOF
end
