# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/


require "pcview.rb"
require APP_ROOT + '/models/invoice'
require APP_ROOT + '/models/payment_amount'
require APP_ROOT + '/models/account_title'
require LIBRARY_DIR + '/datesup'


# 支払済みでない請求書の一覧を表示
class InvoiceList < Wafu::Action
  def execute af
    if af.lines == 0
      app.set_flash "alert", "未払いの請求書は1件もありません。"
      app.redirect_to "invoices-list.rb"
      return
    end

    begin_day, end_day = app.user.user_setting.get_addable_period
    model = {
      :period => "#{begin_day} 〜 #{end_day}",
      :sel_date => get_sel_date("", nil, af.mon, af.day),
      :sel_cash_ac => get_sel_active_items(@app.user, "cash", af.cash_ac.to_i) {|item| 
                                             item.cf_type == 4},
      :list => [],
      :notice => @app.get_flash("notice"),
      :id => af.payment ? af.payment.id : nil,    # 編集の場合
      :action_url => af.payment ? "payment-edit.rb" : "credit.rb"
    }
    if af.error?
      model[:error] = {:message => af.errors}
    end

    model[:lines] = af.lines
    (0..af.lines - 1).each {|i|
      r = af.data[i][:invoice]
      line_hash = {
        :date => r.date, # 購入日
        :partner => r.partner.name,
        :invname => "inv#{i}",
        :invid => r.id,
        :desc => r.journal_entries.first.comment,
        :amount => decimal_format(r.amount_total),
        :paid => decimal_format(af.data[i][:paid].to_i),
        :remain => decimal_format(r.amount_total - af.data[i][:paid].to_i),
        :payname => "pay#{i}",
        :payv => af.data[i][:pay]   # 編集の場合
      }
      model[:list] << line_hash
    }

    @app.page_out("payment_new.html", model)
  end
end

# 
class PaymentAF < Wafu::ActionForm
  def validate
    @mon = @app.cgi.get('mon')
    @day = @app.cgi.get('day')
    @cash_ac = @app.cgi.get('cash')
    @lines = @app.cgi.get('lines').to_i
    raise "internal error" if @lines == 0
    @payment = Payment.find(@app.cgi.get('id')) if @app.cgi.get('id', "") != ""

    err = []
    begin_day, end_day = app.user.user_setting.get_addable_period
    d = get_full_day(@mon.to_i, @day.to_i, begin_day)
    err << "日付が不正です。" if !d
    err << "登録可能期間より前の日付は登録できません。" if d < begin_day
    err << "#{d}は登録可能期間より新しい日付です。メニューより締め処理をおこなってください。" if d > end_day

    # 相手ごとにマイナスの支払いは禁止。（請求書単位でのマイナスは許可）
    @data = []
    amount = {}
    count = 0
    (0..lines - 1).each {|i|
      inv = Invoice.find(@app.cgi.get("inv#{i}"))
      payv = @app.cgi.get("pay#{i}")
      @data << {:invoice => inv, :paid => inv.get_paid, :pay => payv}
      if payv.to_i != 0
        amount[inv.partner.id] = (amount[inv.partner.id] || 0) + payv.to_i
        count += 1

        if payv.to_i > inv.amount_total - inv.get_paid
          err << "#{i+1}行目: 残額より支払額のほうが多くなっています。"
        end
      end
    }
    err << "1件も支払いがありません。" if count == 0
    amount.each {|partner_id, amount|
      err << "#{Partner.find(partner_id).name}に対する支払いがマイナスです。" if amount < 0
    }

    return set_error(err)
  end
  attr_reader :mon, :day, :cash_ac, :lines, :data
  attr_reader :payment
end
