#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 債務の消し込み（編集）


require 'payment-common.rb'


class FirstData < Wafu::ActionForm
  def setup
    pa = PaymentAmount.find @app.cgi.get('id')
    @payment = pa.payment
    je = JournalEntry.find :first, 
              :conditions => ['user_id = ? AND payment_amount_id = ?', 
                              @app.user.id, pa.id]
    raise if !je

    @mon = @payment.date.mon
    @day = @payment.date.day
    @cash_ac = je.cr_code

    @data = []
    @payment.invoices.each {|inv|
      pay = @payment.payment_amounts.find_by_invoice_id(inv).amount
      @data << {
        :invoice => inv,
        :paid => inv.get_paid.to_i - pay,
        :pay => pay
      }
    }
    @lines = @data.size
  end
  attr_reader :mon, :day, :cash_ac, :lines, :data
  attr_reader :payment
end


# 更新
class PaymentUpdate < Wafu::Action
  def execute(af)
    raise if !af.payment.is_a?(Payment)

    user_setting = @app.user.user_setting
    begin_day, end_day = user_setting.get_addable_period
    d = get_full_day(af.mon.to_i, af.day.to_i, begin_day)
    raise if !d || d < begin_day || d > end_day

    Payment.transaction {
      # 登録可能期間の分のみ編集可能なので、
      # (1) いったん関連する支払明細、仕訳を取り消して、
      JournalEntry.delete_all [<<EOF, @app.user.id, af.payment.id]
user_id = ? AND payment_amount_id in 
        (SELECT id FROM payment_amounts WHERE payment_id = ?)
EOF
      PaymentAmount.delete_all ['payment_id = ?', af.payment.id]

      # (2) 作り直す
      af.payment.date = d
      af.payment.save!

      entry_no = (user_setting.entry_no_counter += 1)
      user_setting.save!
      cash_ac = AccountTitle.find(af.cash_ac)
      af.data.each {|line|
        if line[:pay].to_i != 0
          inv = line[:invoice]
          raise if inv.user.id != @app.user.id
          pa = PaymentAmount.new(:payment => af.payment, 
                                 :invoice => inv, 
                                 :amount => line[:pay].to_i)
          pa.save!
          af.payment.payment_amounts << pa

          e = JournalEntry.new(:user => @app.user,
                               :date => d,
                               :entry_no => entry_no,
                               :dr_code => user_setting.payable_item,
                               :cr_code => cash_ac.id,
                               :amount => pa.amount,
                               :remarks => inv.partner.name,
                               :comment => "支払い " + inv.journal_entries.first.comment,
                               :payment_amount => pa)
          e.save!
        end
      }
      raise if af.payment.payment_amounts.size == 0 # 1件もない
    }

    # @app.set_flash("notice", "支払いを登録しました。")
    @app.page_out("payment_create.html", {})
  end
end


PCView.new('actions' => {
  "*" => [FirstData, InvoiceList, nil],
  "pay" => [PaymentAF, PaymentUpdate, InvoiceList]
}).run()
