#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

require "pcview.rb"

class AgreeAF < Wafu::ActionForm
  attr_reader :checked
  attr_reader :userid, :realname, :mail
  
  def setup()
    @checked = false
  end

  def validate()
    ret = []
    if @app.cgi.get("un") == ""
      ret << "「利用規約を読んだ」チェックボックスにチェックが入っていません。"
    end
    return set_error(ret)
  end
end

class Registration < Wafu::Action
  def execute(af)
    model = {
      :username => af.userid,
      :realname => af.realname,
      :mail => af.mail,
      :from_addr => FROM_ADDR
    }
    if af.error?
      model[:errorlist] = af.errors
    end

    @app.page_out("signup2.html", model)
  end
end

class RegAF < Wafu::ActionForm
  attr_reader :userid, :realname, :mail

  def validate()
    @userid = @app.cgi.get("username")
    @realname = @app.cgi.get("realname")
    @mail = @app.cgi.get("mail")

    # expire sweep
    User.find(:all, :conditions => ["expire < ?", Time.now]).each do |ent|
      User.delete ent.id
    end

    ret = []
    if @userid == ""
      ret << "ユーザIDがありません。"
    elsif @userid =~ /[^a-zA-Z0-9]/
      ret << "ユーザIDに半角アルファベット・数字以外が含まれています。"
    else
      if User.find_by_login(@userid)
        ret << "すでに同じユーザIDが使われています。"
      end
    end

    if @realname == ""
      ret << "お名前がありません。"
    end

    if @mail == ""
      ret << "メールアドレスがありません。"
    elsif @mail =~ /[^a-zA-Z0-9\.\-@_]/ || @mail !~ /@/
      ret << "不正なメールアドレスです。"
    end

    return set_error(ret)
  end
end


# 仮登録完了
class RegDone < Wafu::Action
  def format_mail(ent, pwd, time)
    header = {"From" => FROM_ADDR,
              "To" => ent.email,
              "MIME-Version" => "1.0",
              "Content-Type" => "text/plain; charset=ISO-2022-JP",
              "Subject" => "#{SITE_NAME}: ■　仮登録完了のお知らせ　■" }

    mailbody = <<EOF
■　仮登録完了のお知らせ　■

#{SITE_NAME}の仮登録が完了しました。

48時間以内に以下のユーザIDとパスワードでログインすれば，正式にユーザー登録されます。

    #{SITE_NAME}    #{SITE_URL}

ユーザID:  #{ent.login}
パスワード: #{pwd}
仮登録日時: #{time.strftime("%Y.%m.%d %H:%M")}（日本時間）
有効期限:   #{ent.expire.strftime("%Y.%m.%d %H:%M")}

この件に関して思い当たる節がないときは，そのまま無視してください。
-----------------------------------------------------------
お問い合わせ先 mailto:#{ADMIN_ADDR}
EOF
    return [header, mailbody]
  end
  
  def format_mail_admin(ent, time)
    subject = "#{SITE_NAME}: ID発行"
    header = {"From" => FROM_ADDR,
              "To" => ADMIN_ADDR,
              "MIME-Version" => "1.0",
              "Content-Type" => "text/plain; charset=ISO-2022-JP",
              "Subject" => subject }

    mailbody = <<EOF
#{subject}

login: #{ent.login}
name: #{ent.name}
email: #{ent.email}
time: #{CGI.rfc1123_date(time)}
host: #{ENV['REMOTE_HOST'] || get_hostname(ENV['REMOTE_ADDR'].dup.untaint)}
EOF
    return [header, mailbody]
  end
  
  def execute(af)
    now = Time.now
    
    pwd = [[rand(2 ** 32), rand(2 ** 32)].pack("L*")].pack("m")[0..7]
    @ent = User.new(
      :login => af.userid, 
      :name => af.realname, 
      :email => af.mail,
      :expire => now + 48 * 60 * 60 )
    @ent.set_password(pwd)
    @ent.save!

    send_mail(*format_mail(@ent, pwd, now))
    send_mail(*format_mail_admin(@ent, now))

    render 'account-sign-up-done'
  end
end

class PrevAF < Wafu::ActionForm
  attr_reader :checked
  def setup
    @checked = true
  end
end


class FirstCheck < Wafu::ActionForm
  attr_reader :checked
  def setup
    @checked = false
  end
end

class TermOfService < Wafu::Action
  def execute(af)
    model = {
      :cancel => "window.location='#{AppConfig[:site_top_url]}';",
      :checked => af.checked ? "checked" : nil
    }
    if af.error?
      model[:errorlist] = af.errors
    end
    @app.page_out("terms_of_service.html", model)
  end
end
  

if __FILE__ == $0
  PCView.new({"session_type" => "none", 
              "actions" => {
    "*" => [FirstCheck, TermOfService, nil],
    "agree" => [AgreeAF, Registration, TermOfService],
    "register" => [RegAF, RegDone, Registration],
    "prev" => [PrevAF, TermOfService, nil]
  }}).run
end
