#!/home/netsphere/bin/ruby
# -*- mode:ruby; coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/


require "pcview.rb"
require "closing-common.rb"
require CONF_DIR + "/item-default.rb"
require APP_ROOT + '/models/account_title'


def set_closing_day af, user
  day = af["day"].to_i
  raise ArgumentError, "day must be 1..31, but day is #{day}" if day < 1 || day > 31

  if !user.user_setting
    user.user_setting = UserSetting.new(:entry_no_counter => 0, 
                                        :closing_day => day)
    user.user_setting.save!
  else
    prof = user.user_setting
    prof.closing_day = day
    prof.save!
  end
end


class InitialTitles < Wafu::Action
  SUB_STEPS = [
     ["現金同等物",   "現金、日常使う預金、日常使える金券類など。クレジットカードは、現金同等物ではなく、負債で登録してください。", 1, 4],
     ["固定資産・投資", "装置など有形固定資産、定期預金・株式などの投資", 1, 2],
     ["その他の資産", "未収入金など将来の入金があるもの", 1, 1],
     ["有利子負債",  "自動車・住宅ローンなど、利子が発生するもの", 2, 3],
     ["その他の負債", "クレジットカードなど、将来の支出になるもの", 2, 1],
     ["純資産",   "資産から負債を差し引いたあとの、純粋な自分の財産。「未処分利益」勘定は自動で追加されるため、設定不要です。", 3, 3],
     ["営業収益", "給料収入、家賃収入など、通常の収入。", 4, 1],
     ["営業費用", "日常の支出。", 5, 1],
     ["その他の収益", "主に、金融収益（利息、キャピタルゲイン）など。", 6, 1],
     ["その他の費用", "臨時・特別な支出。", 7, 1],
  ]

  def execute af
    if af["action"] != "prev"
      # 月末日の設定
      set_closing_day af, app.user
    end

    # 未処分利益は特別扱い
    un_id = app.user.user_setting.unappropriated_item
    if !un_id
      @ac_list = get_initial_titles()
    else
      # 次のページから戻ってきたとき
      @ac_list = current_titles un_id
      # @ap_sort = app.user.user_setting.payable_item.sort
      # raise "ap.sort '#{@ap_sort}' missing!" if !@ap_sort
    end

    render "welcome-titles", :layout => "application"
  end

private
  # 初期設定
  def get_initial_titles()
    r = Array.new
    DEFAULT_ITEMS.each do |block|
      ary = []
      block[:items].each do |e|
        ary << AccountTitle.new(
          :sort => e[0], :name => e[1], :name_short => e[2], 
          :ac_type => e[3], :cf_type => e[4], :description => e[5])
      end
      # 予備
      for i in 0..2 
        ary << AccountTitle.new(:ac_type => block[:ac_type], 
                                :cf_type => block[:cf_type])
      end
      r << ary
    end

    return r
  end

  # 現在の設定 (前のページから戻ったとき用)
  def current_titles un_id
    raise "un '#{un_id}' missing!" if !un_id

    r = []
    SUB_STEPS.each do |block|
      r << AccountTitle.find(:all, 
                :conditions => [<<EOF, app.user.id, block[2], block[3], un_id])
user_id = ? AND ac_type = ? AND cf_type = ? AND id <> ?
EOF
      for i in 0..2 
        r.last << AccountTitle.new(:ac_type => block[2], :cf_type => block[3])
      end
    end

    return r
  end

=begin
  def group_desc(group)
    case group
    when 1
      "資産は，現金・預金などの財産や，未収入金など将来の入金となるものです。現金や通常使う預金はC/F区分を現金同等物、定期預金などは投資活動にします。"
    when 2
      "負債は，ローン，クレジットカードの残高など，将来何らかの支出（返済）が必要なものです。"
    when 3
      "純資産は，資産から負債を差し引いた，純粋な自身の財産です。"
    when 4
      "営業収益は，通常の給料、家賃収入など，経常的な収入です。"
    when 5
      "営業費用は，経常的に発生する支出です。金融費用、税金、異常な損失は含みません。"
    when 6
      "その他の収益は，金融収益（利息など），臨時的な収入です。"
    when 7
      "その他の費用は，金融費用，税金，臨時的な支出です。"
    end
  end
=end

end


(PCView.new "actions" => {
  "*" => [Wafu::ActionForm, InitialTitles, nil]
}).run
